﻿//-------------------------------------------------------------------------------//
//Copyright (c) 2016, Vincent Bengtsson
//All rights reserved.

//Redistribution and use in source and binary forms, with or without
//modification, are permitted provided that the following conditions are met:

//1. Redistributions of source code must retain the above copyright notice, this
//   list of conditions and the following disclaimer. 
//2. Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.

//THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//The views and conclusions contained in the software and documentation are those
//of the authors and should not be interpreted as representing official policies, 
//either expressed or implied, of the FreeBSD Project.
//-------------------------------------------------------------------------------//
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Diagnostics;
using System.Net.Sockets;
using System.Net;

namespace CSharp_TCP_Messaging_System
{
    public partial class ServerWindow : System.Windows.Forms.Form
    {

        TcpListener Listener;
        private ExtendedTcpClient _client;
        public ExtendedTcpClient Client
        {
            get { return _client; }
            set
            {
                if (_client != null)
                {
                    _client.PacketReceived -= Client_PacketReceived;
                }
                _client = value;
                if (_client != null)
                {
                    _client.PacketReceived += Client_PacketReceived;
                }
            }
        }

        private void Button1_Click(System.Object sender, System.EventArgs e)
        {
            Listener = new TcpListener(IPAddress.Parse("0.0.0.0"), (int)NumericUpDown1.Value);
            Listener.Start();
            ListenerTimer.Start();
            NumericUpDown1.Enabled = false;
            Button1.Enabled = false;
        }

        private void ListenerTimer_Tick(System.Object sender, System.EventArgs e)
        {
            if (Listener.Pending() == true)
            {
                Client = new ExtendedTcpClient(this);
                TcpClient underlyingClient = Listener.AcceptTcpClient();
                Client.SetNewClient(ref underlyingClient);
            }
        }

        private void Client_PacketReceived(object sender, ExtendedTcpClient.PacketReceivedEventArgs e)
        {
            if (e.Packet.Header == TcpMessagePacket.PacketHeader.PlainText)
            {
                TextBox1.AppendText("Message recieved: " + System.Text.Encoding.Default.GetString(e.Packet.Contents) + Environment.NewLine);
            }
        }

        private void ServerWindow_FormClosing(object sender, System.Windows.Forms.FormClosingEventArgs e)
        {
            if (Client != null)
                Client.Disconnect();
            if (Listener != null)
                Listener.Stop();
        }

        private void ServerWindow_Load(System.Object sender, System.EventArgs e)
        {
            ClientWindow client = new ClientWindow();
            client.Show();
        }

        public ServerWindow()
        {
            InitializeComponent();
            Load += ServerWindow_Load;
            FormClosing += ServerWindow_FormClosing;
        }
    }
}